@@ -1,10 +1,11 @@
 import logging
-from time import time
-from typing import Tuple, Callable, Union, List
-from numpy.linalg import norm
-import numpy as np
-from sklearn.neighbors import NearestNeighbors
 import string
+from time import time
+from typing import Callable, List, Optional, Tuple, Union
+
+import numpy as np
+from numpy.linalg import norm
+from sklearn.neighbors import NearestNeighbors
 
 logger = logging.getLogger(__name__)
 
@@ -15,15 +16,15 @@ def _linear_superposition(alphas, vecs, shape):
     Parameters
     ----------
     alphas
-        Coefficients of the superposition
+        Coefficients of the superposition.
     vecs
-        Tensors of the superposition
+        Tensors of the superposition.
     shape
-        Shape of each tensor
+        Shape of each tensor.
 
     Returns
     -------
-    Linear tensor superposition
+    Linear tensor superposition.
     """
     input_str = string.ascii_lowercase[2: 2 + len(shape)]
     einstr = 'a,ba{}->b{}'.format(input_str, input_str)
@@ -38,19 +39,19 @@ def _calculate_global_linearity(predict_fn: Callable, input_shape: Tuple, X_samp
     Parameters
     ----------
     predict_fn
-        Model prediction function
+        Model prediction function.
     input_shape
-        Shape of the input
+        Shape of the input.
     X_samples
-        Array of feature vectors in the linear superposition
+        Array of feature vectors in the linear superposition.
     model_type
-        'classifier' or 'regressor'
+        Supported values: ``'classifier'`` | ``'regressor'``.
     alphas
-        Array of coefficients in the linear superposition
+        Array of coefficients in the linear superposition.
 
     Returns
     -------
-    Linearity score
+    Linearity score.
 
     """
     ss = X_samples.shape[:2]  # X_samples shape=(nb_instances, nb_samples, nb_features)
@@ -104,27 +105,27 @@ def _calculate_global_linearity(predict_fn: Callable, input_shape: Tuple, X_samp
 
 def _calculate_pairwise_linearity(predict_fn: Callable, x: np.ndarray, input_shape: Tuple, X_samples: np.ndarray,
                                   model_type: str, alphas: np.ndarray) -> np.ndarray:
-    """Calculates the norm of the difference between the output of a linear superposition of a test vector x and
-    vectors in X_samples and the linear superposition of the outputs, averaged over all the vectors in X_samples.
+    """Calculates the norm of the difference between the output of a linear superposition of a test vector `x` and
+    vectors in `X_samples` and the linear superposition of the outputs, averaged over all the vectors in `X_samples`.
 
     Parameters
     ----------
     predict_fn
-        Model prediction function
+        Model prediction function.
     x
-        Test instance for which to calculate the linearity measure
+        Test instance for which to calculate the linearity measure.
     input_shape
-        Shape of the input
+        Shape of the input.
     X_samples
-        Array of feature vectors in the linear superposition
+        Array of feature vectors in the linear superposition.
     model_type
-        'classifier' or 'regressor'
+        Supported values: ``'classifier'`` | ``'regressor'``.
     alphas
-        Array of coefficients in the linear superposition
+        Array of coefficients in the linear superposition.
 
     Returns
     -------
-    Linearity score
+    Linearity score.
 
     """
     ss = X_samples.shape[:2]  # X_samples shape=(nb_instances, nb_samples, nb_features)
@@ -189,7 +190,7 @@ def _sample_knn(x: np.ndarray, X_train: np.ndarray, nb_samples: int = 10) -> np.
     Parameters
     ----------
     x
-        Central instance for sampling
+        Central instance for sampling.
     X_train
         Training set.
     nb_samples
@@ -197,7 +198,7 @@ def _sample_knn(x: np.ndarray, X_train: np.ndarray, nb_samples: int = 10) -> np.
 
     Returns
     -------
-    Sampled vectors
+    Sampled vectors.
 
     """
     x = x.reshape(x.shape[0], -1)
@@ -218,17 +219,17 @@ def _sample_knn(x: np.ndarray, X_train: np.ndarray, nb_samples: int = 10) -> np.
     return np.asarray(X_sampled)  # shape=(nb_instances, nb_samples, nb_features)
 
 
-def _sample_grid(x: np.ndarray, feature_range: np.ndarray = None, epsilon: float = 0.04,
+def _sample_grid(x: np.ndarray, feature_range: np.ndarray, epsilon: float = 0.04,
                  nb_samples: int = 10, res: int = 100) -> np.ndarray:
-    """Samples data points uniformly from an interval centered at x and with size epsilon * Delta,
-    with delta = f_max - f_min the features ranges.
+    """Samples data points uniformly from an interval centered at `x` and with size `epsilon * delta`,
+    with `delta = f_max - f_min` the features ranges.
 
     Parameters
     ----------
     x
         Instance of interest.
     feature_range
-        Array with min and max values for each feature
+        Array with min and max values for each feature.
     epsilon
         Size of the sampling region around central instance as percentage of features range.
     nb_samples
@@ -236,7 +237,7 @@ def _sample_grid(x: np.ndarray, feature_range: np.ndarray = None, epsilon: float
 
     Returns
     -------
-    Sampled vectors
+    Sampled vectors.
 
     """
     nb_instances = x.shape[0]
@@ -259,11 +260,18 @@ def _sample_grid(x: np.ndarray, feature_range: np.ndarray = None, epsilon: float
     return X_sampled
 
 
-def _linearity_measure(predict_fn: Callable, x: np.ndarray, X_train: np.ndarray = None,
-                       feature_range: Union[List, np.ndarray] = None, method: str = None,
-                       epsilon: float = 0.04, nb_samples: int = 10, res: int = 100,
-                       alphas: np.ndarray = None, model_type: str = 'classifier', agg: str = 'global') -> np.ndarray:
-    """Calculate the linearity measure of the model around an instance of interest x.
+def _linearity_measure(predict_fn: Callable,
+                       x: np.ndarray,
+                       X_train: Optional[np.ndarray] = None,
+                       feature_range: Optional[Union[List, np.ndarray]] = None,
+                       method: Optional[str] = None,
+                       epsilon: float = 0.04,
+                       nb_samples: int = 10,
+                       res: int = 100,
+                       alphas: Optional[np.ndarray] = None,
+                       model_type: str = 'classifier',
+                       agg: str = 'global') -> np.ndarray:
+    """Calculate the linearity measure of the model around an instance of interest `x`.
 
     Parameters
     ----------
@@ -276,7 +284,7 @@ def _linearity_measure(predict_fn: Callable, x: np.ndarray, X_train: np.ndarray
     feature_range
         Array with min and max values for each feature.
     method
-        Method for sampling. Supported values 'knn' or 'grid'.
+        Method for sampling. Supported values: ``'knn'`` | ``'grid'``.
     epsilon
         Size of the sampling region around the central instance as a percentage of feature range.
     nb_samples
@@ -286,13 +294,13 @@ def _linearity_measure(predict_fn: Callable, x: np.ndarray, X_train: np.ndarray
     alphas
         Array of coefficients in the superposition.
     model_type
-        Type of task. Supported values are 'regressor' or 'classifier'.
+        Type of task. Supported values: ``'regressor'`` | ``'classifier'``.
     agg
-        Aggregation method. Supported values are 'global' or 'pairwise'.
+        Aggregation method. Supported values: ``'global'`` | ``'pairwise'``.
 
     Returns
     -------
-    Linearity score
+    Linearity score.
 
     """
     input_shape = x.shape[1:]
@@ -331,27 +339,33 @@ def _infer_feature_range(X_train: np.ndarray) -> np.ndarray:
     Parameters
     ----------
     X_train
-        Training set
+        Training set.
 
     Returns
     -------
-    Feature range
+    Feature range.
     """
     X_train = X_train.reshape(X_train.shape[0], -1)
     return np.vstack((X_train.min(axis=0), X_train.max(axis=0))).T
 
 
-class LinearityMeasure(object):
+class LinearityMeasure:
 
-    def __init__(self, method: str = 'grid', epsilon: float = 0.04, nb_samples: int = 10, res: int = 100,
-                 alphas: np.ndarray = None, model_type: str = 'classifier', agg: str = 'pairwise',
+    def __init__(self,
+                 method: str = 'grid',
+                 epsilon: float = 0.04,
+                 nb_samples: int = 10,
+                 res: int = 100,
+                 alphas: Optional[np.ndarray] = None,
+                 model_type: str = 'classifier',
+                 agg: str = 'pairwise',
                  verbose: bool = False) -> None:
         """
 
         Parameters
         ----------
         method
-            Method for sampling. Supported methods are 'knn' or 'grid'.
+            Method for sampling. Supported methods: ``'knn'`` | ``'grid'``.
         epsilon
             Size of the sampling region around the central instance as a percentage of the features range.
         nb_samples
@@ -361,9 +375,9 @@ class LinearityMeasure(object):
         alphas
             Coefficients in the superposition.
         agg
-            Aggregation method. Supported values are 'global' or 'pairwise'.
+            Aggregation method. Supported values: ``'global'`` | ``'pairwise'``.
         model_type
-            Type of task. Supported values are 'regressor' or 'classifier'.
+            Type of task. Supported values: ``'regressor'`` | ``'classifier'``.
         """
         self.method = method
         self.epsilon = epsilon
@@ -381,11 +395,8 @@ class LinearityMeasure(object):
         Parameters
         ----------
         X_train
-            Training set
+            Training set.
 
-        Returns
-        -------
-        None
         """
         self.X_train = X_train
         self.feature_range = _infer_feature_range(X_train)
@@ -398,13 +409,13 @@ class LinearityMeasure(object):
         Parameters
         ----------
         predict_fn
-            Prediction function
+            Prediction function.
         x
-            Instance of interest
+            Instance of interest.
 
         Returns
         -------
-        Linearity measure
+        Linearity measure.
 
         """
         input_shape = x.shape[1:]
@@ -430,9 +441,16 @@ class LinearityMeasure(object):
         return lin
 
 
-def linearity_measure(predict_fn: Callable, x: np.ndarray, feature_range: Union[List, np.ndarray] = None,
-                      method: str = 'grid', X_train: np.ndarray = None, epsilon: float = 0.04,
-                      nb_samples: int = 10, res: int = 100, alphas: np.ndarray = None, agg: str = 'global',
+def linearity_measure(predict_fn: Callable,
+                      x: np.ndarray,
+                      feature_range: Optional[Union[List, np.ndarray]] = None,
+                      method: str = 'grid',
+                      X_train: Optional[np.ndarray] = None,
+                      epsilon: float = 0.04,
+                      nb_samples: int = 10,
+                      res: int = 100,
+                      alphas: Optional[np.ndarray] = None,
+                      agg: str = 'global',
                       model_type: str = 'classifier') -> np.ndarray:
     """Calculate the linearity measure of the model around an instance of interest x.
 
@@ -445,7 +463,7 @@ def linearity_measure(predict_fn: Callable, x: np.ndarray, feature_range: Union[
     feature_range
         Array with min and max values for each feature.
     method
-        Method for sampling. Supported values 'knn' or 'grid'.
+        Method for sampling. Supported values: ``'knn'`` | ``'grid'``.
     X_train
         Training set.
     epsilon
@@ -457,13 +475,13 @@ def linearity_measure(predict_fn: Callable, x: np.ndarray, feature_range: Union[
     alphas
         Coefficients in the superposition.
     agg
-        Aggregation method. Supported values 'global' or 'pairwise'.
+        Aggregation method. Supported values: ``'global'`` | ``'pairwise'``.
     model_type
-        Type of task. Supported values 'regressor' or 'classifier'.
+        Type of task. Supported values: ``'regressor'`` | ``'classifier'``.
 
     Returns
     -------
-    Linearity measure
+    Linearity measure.
 
     """
     if method == 'knn':
