@@ -1,18 +1,19 @@
 import copy
 import logging
+from functools import partial
+from typing import Any, Callable, Dict, List, Optional, Tuple, Type, Union
 
 import numpy as np
+from skimage.segmentation import felzenszwalb, quickshift, slic
 
-from functools import partial
-from typing import Any, Callable, List, Union, Tuple, Type
-
-from alibi.utils.wrappers import ArgmaxTransformer
+from alibi.api.defaults import DEFAULT_DATA_ANCHOR_IMG, DEFAULT_META_ANCHOR
 from alibi.api.interfaces import Explainer, Explanation
-from alibi.api.defaults import DEFAULT_META_ANCHOR, DEFAULT_DATA_ANCHOR_IMG
-from alibi.exceptions import AlibiPredictorCallException, AlibiPredictorReturnTypeError
+from alibi.exceptions import (AlibiPredictorCallException,
+                              AlibiPredictorReturnTypeError)
+from alibi.utils.wrappers import ArgmaxTransformer
+
 from .anchor_base import AnchorBaseBeam
 from .anchor_explanation import AnchorExplanation
-from skimage.segmentation import felzenszwalb, slic, quickshift
 
 logger = logging.getLogger(__name__)
 
@@ -20,7 +21,7 @@ DEFAULT_SEGMENTATION_KWARGS = {
     'felzenszwalb': {},
     'quickshift': {},
     'slic': {'n_segments': 10, 'compactness': 10, 'sigma': .5}
-}
+}  # type: Dict[str, Dict]
 
 
 def scale_image(image: np.ndarray, scale: tuple = (0, 255)) -> np.ndarray:
@@ -54,7 +55,7 @@ class AnchorImageSampler:
             segmentation_fn: Callable,
             custom_segmentation: bool,
             image: np.ndarray,
-            images_background: np.ndarray = None,
+            images_background: Optional[np.ndarray] = None,
             p_sample: float = 0.5,
             n_covered_ex: int = 10,
     ):
@@ -64,7 +65,7 @@ class AnchorImageSampler:
         Parameters
         ----------
         predictor
-            A callable that takes a tensor of N data points as inputs and returns N outputs.
+            A callable that takes a `numpy` array of `N` data points as inputs and returns `N` outputs.
         segmentation_fn
             Function used to segment the images.
         image
@@ -75,7 +76,7 @@ class AnchorImageSampler:
             Probability for a pixel to be represented by the average value of its superpixel.
         n_covered_ex
             How many examples where anchors apply to store for each anchor sampled during search
-            (both examples where prediction on samples agrees/disagrees with desired_label are stored).
+            (both examples where prediction on samples agrees/disagrees with `desired_label` are stored).
         """
         self.predictor = predictor
         self.segmentation_fn = segmentation_fn
@@ -100,28 +101,35 @@ class AnchorImageSampler:
         Parameters
         ----------
         anchor
-            int: order of anchor in the batch
-            tuple: features (= superpixels) present in the proposed anchor
+            - ``int`` - order of anchor in the batch.
+            - ``tuple`` - features (= superpixels) present in the proposed anchor.
         num_samples
-            Number of samples used
+            Number of samples used.
         compute_labels
-            If True, an array of comparisons between predictions on perturbed samples and
+            If ``True``, an array of comparisons between predictions on perturbed samples and
             instance to be explained is returned.
 
         Returns
         -------
-            If compute_labels=True, a list containing the following is returned:
-                - covered_true: perturbed examples where the anchor applies and the model prediction
-                    on perturbed is the same as the instance prediction
-                - covered_false: perturbed examples where the anchor applies and the model prediction
-                    on pertrurbed sample is NOT the same as the instance prediction
-                - labels: num_samples ints indicating whether the prediction on the perturbed sample
-                    matches (1) the label of the instance to be explained or not (0)
-                - data: Matrix with 1s and 0s indicating whether the values in a superpixel will
-                    remain unchanged (1) or will be perturbed (0), for each sample
-                - 1.0: indicates exact coverage is not computed for this algorithm
-                - anchor[0]: position of anchor in the batch request
-            Otherwise, a list containing the data matrix only is returned.
+        If ``compute_labels=True``, a list containing the following is returned
+
+         - `covered_true` - perturbed examples where the anchor applies and the model prediction  on perturbed is the \
+         same as the instance prediction.
+
+         - `covered_false` - perturbed examples where the anchor applies and the model prediction on pertrurbed sample \
+         is NOT the same as the instance prediction.
+
+         - `labels` - `num_samples` ints indicating whether the prediction on the perturbed sample matches (1) \
+         the label of the instance to be explained or not (0).
+
+         - `data` - Matrix with 1s and 0s indicating whether the values in a superpixel will remain unchanged (1) or \
+         will be perturbed (0), for each sample.
+
+         - `1.0` - indicates exact coverage is not computed for this algorithm.
+
+         - `anchor[0]` - position of anchor in the batch request
+
+        Otherwise, a list containing the data matrix only is returned.
         """
 
         if compute_labels:
@@ -153,7 +161,7 @@ class AnchorImageSampler:
 
         Returns
         -------
-            A boolean array indicating whether the prediction was the same as the instance label.
+        A boolean array indicating whether the prediction was the same as the instance label.
         """
 
         return self.predictor(samples) == self.instance_label
@@ -206,28 +214,28 @@ class AnchorImageSampler:
         Returns
         -------
         imgs
-            A [num_samples, H, W, C] array of perturbed images.
+            A `[num_samples, H, W, C]` array of perturbed images.
         segments_mask
-            A [num_samples, M] binary mask, where M is the number of image superpixels
+            A `[num_samples, M]` binary mask, where `M` is the number of image superpixels
             segments. 1 indicates the values in that particular superpixels are not
             perturbed.
         """
 
         image = self.image
         segments = self.segments
+        backgrounds: Union[np.ndarray, List[None]]
 
         # choose superpixels to be perturbed
         segments_mask = self._choose_superpixels(num_samples, p_sample=self.p_sample)
         segments_mask[:, anchor] = 1
 
         # for each sample, need to sample one of the background images if provided
-        if self.images_background:
+        if self.images_background is not None:
             backgrounds = np.random.choice(
                 range(len(self.images_background)),
                 segments_mask.shape[0],
                 replace=True,
             )
-            segments_mask = np.hstack((segments_mask, backgrounds.reshape(-1, 1)))
         else:
             backgrounds = [None] * segments_mask.shape[0]
             # create fudged image where the pixel value in each superpixel is set to the
@@ -247,13 +255,11 @@ class AnchorImageSampler:
             mask = np.zeros(segments.shape).astype(bool)
             for superpixel in to_perturb:
                 mask[segments == superpixel] = True
-            if background_idx:
+            if background_idx is not None:
                 # replace values with those of background image
-                # TODO: Could images_background be None herre?
-                temp[mask] = self.images_background[background_idx][mask]
+                temp[mask] = self.images_background[background_idx][mask]  # type: ignore[index]
             else:
                 # ... or with the averaged superpixel value
-                # TODO: Where is fudged_image defined?
                 temp[mask] = fudged_image[mask]
             pert_imgs.append(temp)
 
@@ -270,7 +276,7 @@ class AnchorImageSampler:
 
         Returns
         -------
-            A [H, W] array of integers. Each integer is a segment (superpixel) label.
+         A `[H, W]` array of integers. Each integer is a segment (superpixel) label.
         """
 
         image_preproc = self._preprocess_img(image)
@@ -288,7 +294,7 @@ class AnchorImageSampler:
 
         Returns
         -------
-            A preprocessed image.
+        A preprocessed image.
         """
 
         # Grayscale images are repeated across channels
@@ -306,26 +312,26 @@ class AnchorImage(Explainer):
                  image_shape: tuple,
                  dtype: Type[np.generic] = np.float32,
                  segmentation_fn: Any = 'slic',
-                 segmentation_kwargs: dict = None,
-                 images_background: np.ndarray = None,
-                 seed: int = None) -> None:
+                 segmentation_kwargs: Optional[dict] = None,
+                 images_background: Optional[np.ndarray] = None,
+                 seed: Optional[int] = None) -> None:
         """
         Initialize anchor image explainer.
 
         Parameters
         ----------
         predictor
-            A callable that takes a tensor of N data points as inputs and returns N outputs.
+            A callable that takes a `numpy` array of `N` data points as inputs and returns `N` outputs.
         image_shape
-            Shape of the image to be explained.
+            Shape of the image to be explained. The channel axis is expected to be last.
         dtype
-            A numpy scalar type that corresponds to the type of input array expected by `predictor`. This may be
+            A `numpy` scalar type that corresponds to the type of input array expected by `predictor`. This may be
             used to construct arrays of the given type to be passed through the `predictor`. For most use cases
             this argument should have no effect, but it is exposed for use with predictors that would break when
             called with an array of unsupported type.
         segmentation_fn
-            Any of the built in segmentation function strings: 'felzenszwalb', 'slic' or 'quickshift' or a custom
-            segmentation function (callable) which returns an image mask with labels for each superpixel.
+            Any of the built in segmentation function strings: ``'felzenszwalb'``, ``'slic'`` or ``'quickshift'`` or
+            a custom segmentation function (callable) which returns an image mask with labels for each superpixel.
             See http://scikit-image.org/docs/dev/api/skimage.segmentation.html for more info.
         segmentation_kwargs
             Keyword arguments for the built in segmentation functions.
@@ -344,9 +350,11 @@ class AnchorImage(Explainer):
         super().__init__(meta=copy.deepcopy(DEFAULT_META_ANCHOR))
         np.random.seed(seed)
 
-        if isinstance(segmentation_fn, str) and not segmentation_kwargs:
+        # TODO: this logic needs improvement. We should check against a fixed set of strings
+        # for built-ins instead of any `str`.
+        if isinstance(segmentation_fn, str) and segmentation_kwargs is None:
             try:
-                segmentation_kwargs = DEFAULT_SEGMENTATION_KWARGS[segmentation_fn]  # type: ignore
+                segmentation_kwargs = DEFAULT_SEGMENTATION_KWARGS[segmentation_fn]
             except KeyError:
                 logger.warning(
                     'DEFAULT_SEGMENTATION_KWARGS did not contain any entry'
@@ -357,9 +365,11 @@ class AnchorImage(Explainer):
         elif callable(segmentation_fn) and segmentation_kwargs:
             logger.warning(
                 'Specified both a segmentation function to create superpixels and '
-                'keyword arguments for built segmentation functions. By default '
+                'keyword arguments for built-in segmentation functions. By default '
                 'the specified segmentation function will be used.'
             )
+        else:
+            segmentation_kwargs = {}
 
         # set the predictor
         self.image_shape = tuple(image_shape)  # coerce lists
@@ -404,7 +414,7 @@ class AnchorImage(Explainer):
 
         Returns
         -------
-            A [H, W] array of integers. Each integer is a segment (superpixel) label.
+        A `[H, W]` array of integers. Each integer is a segment (superpixel) label.
         """
 
         image_preproc = self._preprocess_img(image)
@@ -422,7 +432,7 @@ class AnchorImage(Explainer):
 
         Returns
         -------
-            A preprocessed image.
+        A preprocessed image.
         """
 
         # Grayscale images are repeated across channels
@@ -433,7 +443,7 @@ class AnchorImage(Explainer):
 
         return image_preproc
 
-    def explain(self,  # type: ignore
+    def explain(self,  # type: ignore[override]
                 image: np.ndarray,
                 p_sample: float = 0.5,
                 threshold: float = 0.95,
@@ -443,7 +453,7 @@ class AnchorImage(Explainer):
                 coverage_samples: int = 10000,
                 beam_size: int = 1,
                 stop_on_first: bool = False,
-                max_anchor_size: int = None,
+                max_anchor_size: Optional[int] = None,
                 min_samples_start: int = 100,
                 n_covered_ex: int = 10,
                 binary_cache_size: int = 10000,
@@ -463,7 +473,7 @@ class AnchorImage(Explainer):
         threshold
             Minimum precision threshold.
         delta
-            Used to compute beta.
+            Used to compute `beta`.
         tau
             Margin between lower confidence bound and minimum precision of upper bound.
         batch_size
@@ -473,7 +483,7 @@ class AnchorImage(Explainer):
         beam_size
             The number of anchors extended at each step of new anchors construction.
         stop_on_first
-            If True, the beam search algorithm will return the first anchor that has satisfies the
+            If ``True``, the beam search algorithm will return the first anchor that has satisfies the
             probability constraint.
         max_anchor_size
             Maximum number of features in result.
@@ -481,12 +491,12 @@ class AnchorImage(Explainer):
             Min number of initial samples.
         n_covered_ex
             How many examples where anchors apply to store for each anchor sampled during search
-            (both examples where prediction on samples agrees/disagrees with desired_label are stored).
+            (both examples where prediction on samples agrees/disagrees with `desired_label` are stored).
         binary_cache_size
-            The result search pre-allocates binary_cache_size batches for storing the binary arrays
+            The result search pre-allocates `binary_cache_size` batches for storing the binary arrays
             returned during sampling.
         cache_margin
-            When only max(cache_margin, batch_size) positions in the binary cache remain empty, a new cache
+            When only ``max(cache_margin, batch_size)`` positions in the binary cache remain empty, a new cache
             of the same size is pre-allocated to continue buffering samples.
         verbose
             Display updates during the anchor search iterations.
@@ -497,6 +507,10 @@ class AnchorImage(Explainer):
         -------
         explanation
             `Explanation` object containing the anchor explaining the instance with additional metadata as attributes.
+            See usage at `AnchorImage examples`_ for details.
+
+            .. _AnchorImage examples:
+                https://docs.seldon.io/projects/alibi/en/latest/methods/Anchors.html
         """
         # get params for storage in meta
         params = locals()
@@ -535,11 +549,11 @@ class AnchorImage(Explainer):
             **kwargs,
         )  # type: Any
 
-        return self.build_explanation(
+        return self._build_explanation(
             image, result, sampler.instance_label, params, sampler
         )
 
-    def build_explanation(
+    def _build_explanation(
             self,
             image: np.ndarray,
             result: dict,
@@ -560,7 +574,7 @@ class AnchorImage(Explainer):
         predicted_label
             Label of the instance to be explained.
         params
-            Parameters passed to `explain`
+            Parameters passed to `:py:meth:alibi.explainers.anchor_image.AnchorImage.explain`.
         """
 
         result['instance'] = image
@@ -598,7 +612,7 @@ class AnchorImage(Explainer):
         image
             Image to be explained.
         segments
-            Superpixels
+            Superpixels.
         mask_features
             List with superpixels present in mask.
         scale
@@ -640,4 +654,12 @@ class AnchorImage(Explainer):
             return transformer
 
     def reset_predictor(self, predictor: Callable) -> None:
+        """
+        Resets the predictor function.
+
+        Parameters
+        ----------
+        predictor
+            New predictor function.
+        """
         self.predictor = self._transform_predictor(predictor)
