@@ -1,35 +1,43 @@
 import copy
 import logging
-import numpy as np
 from collections import OrderedDict, defaultdict
 from itertools import accumulate
-from typing import Any, Callable, DefaultDict, Dict, List, Set, Tuple, Type, Union, Optional
+from typing import (Any, Callable, DefaultDict, Dict, List, Optional, Set,
+                    Tuple, Type, Union)
 
+import numpy as np
+
+from alibi.api.defaults import DEFAULT_DATA_ANCHOR, DEFAULT_META_ANCHOR
 from alibi.api.interfaces import Explainer, Explanation, FitMixin
-from alibi.api.defaults import DEFAULT_META_ANCHOR, DEFAULT_DATA_ANCHOR
-from alibi.exceptions import AlibiPredictorCallException, AlibiPredictorReturnTypeError
-from .anchor_base import AnchorBaseBeam, DistributedAnchorBaseBeam
-from .anchor_explanation import AnchorExplanation
-from alibi.utils.wrappers import ArgmaxTransformer
+from alibi.exceptions import (AlibiPredictorCallException,
+                              AlibiPredictorReturnTypeError)
 from alibi.utils.discretizer import Discretizer
 from alibi.utils.distributed import RAY_INSTALLED
 from alibi.utils.mapping import ohe_to_ord, ord_to_ohe
+from alibi.utils.wrappers import ArgmaxTransformer
+
+from .anchor_base import AnchorBaseBeam, DistributedAnchorBaseBeam
+from .anchor_explanation import AnchorExplanation
 
 
 class TabularSampler:
     """ A sampler that uses an underlying training set to draw records that have a subset of features with
-    values specified in an instance to be expalined, X."""
+    values specified in an instance to be explained, `X`. """
+
+    # if documented in the Attributes, it will be documented twice.
+    # Probably related to: https://github.com/sphinx-doc/sphinx/issues/7427
+    instance_label: int  #: The label of the instance to be explained.
 
     def __init__(self, predictor: Callable, disc_perc: Tuple[Union[int, float], ...], numerical_features: List[int],
                  categorical_features: List[int], feature_names: list, feature_values: dict, n_covered_ex: int = 10,
-                 seed: int = None) -> None:
+                 seed: Optional[int] = None) -> None:
         """
         Parameters
         ----------
         predictor
-            A callable that takes a tensor of N data points as inputs and returns N outputs.
+            A callable that takes a tensor of `N` data points as inputs and returns `N` outputs.
         disc_perc
-            Percentiles used for numerical feat. discretisation.
+            Percentiles used for numerical feature discretisation.
         numerical_features
             Numerical features column IDs.
         categorical_features
@@ -47,7 +55,6 @@ class TabularSampler:
 
         np.random.seed(seed)
 
-        self.instance_label = None  # type: int
         self.predictor = predictor
         self.n_covered_ex = n_covered_ex
 
@@ -64,20 +71,19 @@ class TabularSampler:
 
     def deferred_init(self, train_data: Union[np.ndarray, Any], d_train_data: Union[np.ndarray, Any]) -> Any:
         """
-        Initialise the Tabular sampler object with data, discretizer, feature statistics and
+        Initialise the tabular sampler object with data, discretizer, feature statistics and
         build an index from feature values and bins to database rows for each feature.
 
         Parameters
         ----------
         train_data:
-            Data from which samples are drawn. Can be a numpy array or a ray future.
+            Data from which samples are drawn. Can be a `numpy` array or a `ray` future.
         d_train_data:
-            Discretized version for training data. Can be a numpy array or a ray future.
+            Discretized version for training data. Can be a `numpy` array or a `ray` future.
 
         Returns
         -------
-            An initialised sampler.
-
+        An initialised sampler.
         """
 
         self._set_data(train_data, d_train_data)
@@ -110,7 +116,7 @@ class TabularSampler:
 
     def _set_numerical_feats_stats(self) -> None:
         """
-        Compute min and max for numerical features so that sampling from this range can be performed if
+        Compute `min` and `max` for numerical features so that sampling from this range can be performed if
         a sampling request has bin that is not in the training data.
         """
 
@@ -125,10 +131,10 @@ class TabularSampler:
         Parameters
         ----------
         X
-             Instance to be explained.
+            Instance to be explained.
         """
 
-        label = self.predictor(X.reshape(1, -1))[0]
+        label = self.predictor(X.reshape(1, -1))[0]  # type: int
         self.instance_label = label
 
     def set_n_covered(self, n_covered: int) -> None:
@@ -179,22 +185,29 @@ class TabularSampler:
         num_samples
             Number of samples used when sampling from training set.
         compute_labels
-            If True, an array of comparisons between predictions on perturbed samples and instance to be
+            If ``True``, an array of comparisons between predictions on perturbed samples and instance to be
             explained is returned.
 
         Returns
         -------
-            If compute_labels=True, a list containing the following is returned:
-             - covered_true: perturbed examples where the anchor applies and the model prediction
-                    on perturbation is the same as the instance prediction
-             - covered_false: perturbed examples where the anchor applies and the model prediction
-                    is NOT the same as the instance prediction
-             - labels: num_samples ints indicating whether the prediction on the perturbed sample
-                    matches (1) the label of the instance to be explained or not (0)
-             - data: Sampled data where ordinal features are binned (1 if in bin, 0 otherwise)
-             - coverage: the coverage of the anchor
-             - anchor[0]: position of anchor in the batch request
-            Otherwise, a list containing the data matrix only is returned.
+        If ``compute_labels=True``, a list containing the following is returned
+
+         - `covered_true` - perturbed examples where the anchor applies and the model prediction \
+         on perturbation is the same as the instance prediction.
+
+         - `covered_false` - perturbed examples where the anchor applies and the model prediction \
+         is NOT the same as the instance prediction.
+
+         - `labels` - `num_samples` ints indicating whether the prediction on the perturbed sample \
+         matches (1) the label of the instance to be explained or not (0).
+
+         - `data` -  Sampled data where ordinal features are binned (1 if in bin, 0 otherwise).
+
+         - `coverage` - the coverage of the anchor.
+
+         - `anchor[0]` - position of anchor in the batch request.
+
+        Otherwise, a list containing the data matrix only is returned.
         """
 
         raw_data, d_raw_data, coverage = self.perturbation(anchor[1], num_samples)
@@ -215,7 +228,8 @@ class TabularSampler:
             labels = self.compare_labels(raw_data)
             covered_true = raw_data[labels, :][:self.n_covered_ex]
             covered_false = raw_data[np.logical_not(labels), :][:self.n_covered_ex]
-            return [covered_true, covered_false, labels.astype(int), data, coverage, anchor[0]]
+            return [covered_true, covered_false, labels.astype(int), data, coverage,
+                    anchor[0]]  # type: ignore[return-value]
         else:
             return [data]  # only binarised data is used for coverage computation
 
@@ -231,14 +245,14 @@ class TabularSampler:
 
         Returns
         -------
-            An array of integers indicating whether the prediction was the same as the instance label.
+        An array of integers indicating whether the prediction was the same as the instance label.
         """
 
         return self.predictor(samples) == self.instance_label
 
     def perturbation(self, anchor: tuple, num_samples: int) -> Tuple[np.ndarray, np.ndarray, float]:
         """
-        Implements functionality described in __call__.
+        Implements functionality described in :py:meth:`alibi.explainers.anchor_tabular.TabularSampler.__call__`.
 
         Parameters
         ----------
@@ -249,11 +263,10 @@ class TabularSampler:
 
         Returns
         -------
-
         samples
             Sampled data from training set.
         d_samples
-            Like samples, but continuous data is converted to oridinal discrete data (binned).
+            Like samples, but continuous data is converted to ordinal discrete data (binned).
         coverage
             The coverage of the result in the training data.
         """
@@ -286,8 +299,9 @@ class TabularSampler:
         num_samples_pos = np.searchsorted(nb_partial_anchors, num_samples)
         if num_samples_pos == 0:
             samples_idxs = np.random.choice(partial_anchor_rows[-1], num_samples)
-            samples[:, uniq_feat_ids] = self.train_data[np.ix_(samples_idxs, uniq_feat_ids)]
-            d_samples[:, uniq_feat_ids] = self.d_train_data[np.ix_(samples_idxs, uniq_feat_ids)]
+            samples[:, uniq_feat_ids] = self.train_data[np.ix_(samples_idxs, uniq_feat_ids)]  # type: ignore[arg-type]
+            d_samples[:, uniq_feat_ids] = self.d_train_data[
+                np.ix_(samples_idxs, uniq_feat_ids)]  # type: ignore[arg-type]
 
             return samples, d_samples, coverage
 
@@ -317,13 +331,13 @@ class TabularSampler:
 
         Parameters
         ----------
-        allowed_bins:
-            See get_feature_index method.
-        num_samples:
+        allowed_bins
+            See :py:meth:`alibi.explainers.anchor_tabular.TabularSampler.get_feature_index` method.
+        num_samples
             Number of replacement values.
-        samples:
+        samples
             Contains the samples whose values are to be replaced.
-        unk_feature_values:
+        unk_feature_values
             List of tuples where: [0] is original feature id, [1] feature type, [2] if var is categorical,
             replacement value, otherwise None
         """
@@ -363,7 +377,7 @@ class TabularSampler:
             the entire anchor applies.
         nb_partial_anchors
             The number of training records which contain each partial anchor.
-        num_samples:
+        num_samples
             Number of perturbed samples to be returned.
         """
 
@@ -375,7 +389,7 @@ class TabularSampler:
 
         # replace partial anchors with partial anchors drawn from the training dataset
         # samp_idxs are arrays of training set row indices from where partial anchors are extracted for replacement
-        for idx, n_samp in enumerate(nb_partial_anchors[start_idx:end_idx + 1], start=start_idx):  # type: ignore
+        for idx, n_samp in enumerate(nb_partial_anchors[start_idx:end_idx + 1], start=start_idx):  # type: ignore[misc]
             if num_samples >= n_samp:
                 samp_idxs = partial_anchor_rows[n_anchor_feats - idx - 1]
                 num_samples -= n_samp
@@ -389,7 +403,8 @@ class TabularSampler:
                         replace=True,
                     )
                 n_samp = num_samples
-            samples[start:start + n_samp, uniq_feat_ids[idx:]] = self.train_data[np.ix_(samp_idxs, uniq_feat_ids[idx:])]
+            samples[start:start + n_samp, uniq_feat_ids[idx:]] = self.train_data[
+                np.ix_(samp_idxs, uniq_feat_ids[idx:])]  # type: ignore[arg-type]
 
             # deal with partial anchors; idx = 0 means that we actually sample the entire anchor
             if idx > 0:
@@ -428,7 +443,7 @@ class TabularSampler:
         the same value as the feature in the instance to be explained (for ordinal variables, the row
         indices are those of rows which contain records with feature values in the same bin). The algorithm
         uses both the feature *encoded* ids in anchor and the feature ids in the input data set. The two
-        are mapped by self.enc2feat_idx.
+        are mapped by `self.enc2feat_idx`.
 
         Parameters
         ----------
@@ -444,15 +459,16 @@ class TabularSampler:
             Maps original feature ids to the training set rows where these features have the same value as the anchor.
         unk_feat_values
             When a categorical variable with the specified value/discretized variable in the specified bin is not found
-            in the training set, a tuple is added to unk_feat_values to indicate the original feature id, its type
-            ('c'=categorical, o='discretized continuous') and the value/bin it should be sampled from.
+            in the training set, a tuple is added to `unk_feat_values` to indicate the original feature id, its type
+            (``'c'`` = categorical, ``'o'`` = discretized continuous) and the value/bin it should be sampled from.
         """
 
         # bins one can sample from for each numerical feature (key: feat id)
         allowed_bins = {}  # type: Dict[int, Set[int]]
         # index of database rows (values) for each feature in result (key: feat id)
         allowed_rows = {}  # type: Dict[int, Any[int]]
-        unk_feat_values = []  # feats for which there are not training records in the desired bin/with that value
+        # feats for which there are not training records in the desired bin/with that value
+        unk_feat_values = []  # type: List[Tuple[int, str, Optional[int]]]
         cat_enc_ids = [enc_id for enc_id in anchor if enc_id in self.cat_lookup.keys()]
         ord_enc_ids = [enc_id for enc_id in anchor if enc_id in self.ord_lookup.keys()]
         if cat_enc_ids:
@@ -485,33 +501,38 @@ class TabularSampler:
     def build_lookups(self, X: np.ndarray) -> List[Dict]:
         """
         An encoding of the feature IDs is created by assigning each bin of a discretized numerical variable and each
-        categorical variable a unique index. For a dataset containg, e.g., a numerical variable with 5 bins and
+        categorical variable a unique index. For a dataset containing, e.g., a numerical variable with 5 bins and
         3 categorical variables, indices 0 - 4 represent bins of the numerical variable whereas indices 5, 6, 7
         represent the encoded indices of the categorical variables (but see note for caviats). The encoding is
         necessary so that the different ranges of the numerical variable can be sampled during result construction.
         Note that the encoded indices represent the predicates used during the anchor construction process (i.e., and
         anchor is a collection of encoded indices.
 
-        Note: Each continuous variable has n_bins - 1 corresponding entries in ord_lookup.
-
         Parameters
         ---------
         X
-            instance to be explained
+            Instance to be explained.
 
         Returns
         -------
-            a list containing three dictionaries, whose keys are encoded feature IDs:
-             - cat_lookup: maps categorical variables to their value in X
-             - ord_lookup: maps discretized numerical variables to the bins they can be sampled from given X
-             - enc2feat_idx: maps the encoded IDs to the original (training set) feature column IDs
+        A list containing three dictionaries, whose keys are encoded feature IDs
+
+         - `cat_lookup` - maps categorical variables to their value in `X`.
+
+         - `ord_lookup` - maps discretized numerical variables to the bins they can be sampled from given `X`.
+
+         - `enc2feat_idx` - maps the encoded IDs to the original (training set) feature column IDs.
+
+        Notes
+        -----
+        Each continuous variable has `n_bins - 1` corresponding entries in `ord_lookup`.
         """
 
         X = self.disc.discretize(X.reshape(1, -1))[0]  # map continuous features to ordinal discrete variables
 
         if not self.numerical_features:  # data contains only categorical variables
             self.cat_lookup = dict(zip(self.categorical_features, X))
-            self.enc2feat_idx = dict(zip(*[self.categorical_features] * 2))  # type: ignore
+            self.enc2feat_idx = dict(zip(*[self.categorical_features] * 2))  # type: ignore[arg-type]
             return [self.cat_lookup, self.ord_lookup, self.enc2feat_idx]
 
         first_numerical_idx = np.searchsorted(self.categorical_features, self.numerical_features[0]).item()
@@ -560,10 +581,11 @@ class TabularSampler:
 
 
 class RemoteSampler:
-    """ A wrapper that facilitates the use of TabularSampler for distributed sampling."""
+    """ A wrapper that facilitates the use of `TabularSampler` for distributed sampling."""
     if RAY_INSTALLED:
         import ray
-        ray = ray  # set module as class variable to used only in this context
+        # set module as class variable to used only in this context
+        ray = ray  #: `ray` module.
 
     def __init__(self, *args):
         self.train_id, self.d_train_id, self.sampler = args
@@ -572,17 +594,14 @@ class RemoteSampler:
     def __call__(self, anchors_batch: Union[Tuple[int, tuple], List[Tuple[int, tuple]]], num_samples: int,
                  compute_labels: bool = True) -> List:
         """
-        Wrapper around TabularSampler.__call__. It allows sampling a batch of anchors in the same process,
-        which can improve performance.
+        Wrapper around :py:meth:`alibi.explainers.anchor_tabular.TabularSampler.__call__`. It allows sampling a batch
+        of anchors in the same process, which can improve performance.
 
         Parameters
         ----------
-        anchors_batch:
-            A list of result tuples. see TabularSampler.__call__ for details.
-        num_samples:
-            See TabularSampler.__call__.
-        compute_labels
-            See TabularSampler.__call__.
+        anchors_batch, num_samples, compute_labels
+            A list of result tuples. See :py:meth:`alibi.explainers.anchor_tabular.TabularSampler.__call__`
+            for details.
         """
 
         if isinstance(anchors_batch, tuple):  # DistributedAnchorBaseBeam._get_samples_coverage call
@@ -634,22 +653,22 @@ class RemoteSampler:
 
         Returns
         -------
-            The tabular sampler object that is used in the process.
+        The tabular sampler object that is used in the process.
         """
         return self.sampler
 
-    def build_lookups(self, X):
+    def build_lookups(self, X: np.ndarray):
         """
-        Wrapper around TabularSampler.build_lookups.
+        Wrapper around :py:meth:`alibi.explainers.anchor_tabular.TabularSampler.build_lookups`.
 
         Parameters
         --------
         X
-            See TabularSampler.build_lookups.
+            See :py:meth:`alibi.explainers.anchor_tabular.TabularSampler.build_lookups`.
 
         Returns
         -------
-            See TabularSampler.build_lookups.
+        See :py:meth:`alibi.explainers.anchor_tabular.TabularSampler.build_lookups`.
         """
 
         cat_lookup_id, ord_lookup_id, enc2feat_idx_id = self.sampler.build_lookups(X)
@@ -658,25 +677,26 @@ class RemoteSampler:
 
 
 class AnchorTabular(Explainer, FitMixin):
+    instance_label: int  #: The label of the instance to be explained.
 
     def __init__(self,
                  predictor: Callable[[np.ndarray], np.ndarray],
                  feature_names: List[str],
-                 categorical_names: Dict[int, List[str]] = None,
+                 categorical_names: Optional[Dict[int, List[str]]] = None,
                  dtype: Type[np.generic] = np.float32,
                  ohe: bool = False,
-                 seed: int = None) -> None:
+                 seed: Optional[int] = None) -> None:
         """
         Parameters
         ----------
         predictor
-            A callable that takes a tensor of N data points as inputs and returns N outputs.
+            A callable that takes a `numpy` array of `N` data points as inputs and returns `N` outputs.
         feature_names
             List with feature names.
         categorical_names
             Dictionary where keys are feature columns and values are the categories for the feature.
         dtype
-            A numpy scalar type that corresponds to the type of input array expected by `predictor`. This may be
+            A `numpy` scalar type that corresponds to the type of input array expected by `predictor`. This may be
             used to construct arrays of the given type to be passed through the `predictor`. For most use cases
             this argument should have no effect, but it is exposed for use with predictors that would break when
             called with an array of unsupported type.
@@ -698,7 +718,7 @@ class AnchorTabular(Explainer, FitMixin):
         self.ohe = ohe
         self.feature_names = feature_names
 
-        if ohe:
+        if ohe and categorical_names:
             self.cat_vars_ord = {col: len(values) for col, values in categorical_names.items()}
             self.cat_vars_ohe = ord_to_ohe(np.zeros((1, len(feature_names))), self.cat_vars_ord)[1]
 
@@ -721,12 +741,13 @@ class AnchorTabular(Explainer, FitMixin):
         self.samplers = []  # type: list
         self.ohe = ohe
         self.seed = seed
-        self.instance_label = None
 
         # update metadata
         self.meta['params'].update(seed=seed)
 
-    def fit(self, train_data: np.ndarray, disc_perc: Tuple[Union[int, float], ...] = (25, 50, 75),  # type:ignore
+    def fit(self,  # type: ignore[override]
+            train_data: np.ndarray,
+            disc_perc: Tuple[Union[int, float], ...] = (25, 50, 75),
             **kwargs) -> "AnchorTabular":
         """
         Fit discretizer to train data to bin numerical features into ordered bins and compute statistics for
@@ -738,7 +759,7 @@ class AnchorTabular(Explainer, FitMixin):
         train_data
             Representative sample from the training data.
         disc_perc
-            List with percentiles (int) used for discretization.
+            List with percentiles (`int`) used for discretization.
         """
 
         # transform one-hot encodings to labels if ohe == True
@@ -750,7 +771,7 @@ class AnchorTabular(Explainer, FitMixin):
         self.feature_values.update(disc.feature_intervals)
 
         sampler = TabularSampler(
-            self._predictor,
+            self._predictor,  # type: ignore[arg-type] # TODO: fix me, ignored as can be None due to saving.py
             disc_perc,
             self.numerical_features,
             self.categorical_features,
@@ -768,11 +789,11 @@ class AnchorTabular(Explainer, FitMixin):
     def _build_sampling_lookups(self, X: np.ndarray) -> None:
         """
         Build a series of lookup tables used to draw samples with feature subsets identical to
-        given subsets of X (see TabularSampler.build_sampling_lookups for details).
+        given subsets of `X` (see TabularSampler.build_sampling_lookups for details).
 
         Parameters
         ----------
-        X:
+        X
             Instance to be explained.
         """
 
@@ -788,7 +809,7 @@ class AnchorTabular(Explainer, FitMixin):
                 coverage_samples: int = 10000,
                 beam_size: int = 1,
                 stop_on_first: bool = False,
-                max_anchor_size: int = None,
+                max_anchor_size: Optional[int] = None,
                 min_samples_start: int = 100,
                 n_covered_ex: int = 10,
                 binary_cache_size: int = 10000,
@@ -797,7 +818,7 @@ class AnchorTabular(Explainer, FitMixin):
                 verbose_every: int = 1,
                 **kwargs: Any) -> Explanation:
         """
-        Explain prediction made by classifier on instance X.
+        Explain prediction made by classifier on instance `X`.
 
         Parameters
         ----------
@@ -806,7 +827,7 @@ class AnchorTabular(Explainer, FitMixin):
         threshold
             Minimum precision threshold.
         delta
-            Used to compute beta.
+            Used to compute `beta`.
         tau
             Margin between lower confidence bound and minimum precision or upper bound.
         batch_size
@@ -816,7 +837,7 @@ class AnchorTabular(Explainer, FitMixin):
         beam_size
             The number of anchors extended at each step of new anchors construction.
         stop_on_first
-            If True, the beam search algorithm will return the first anchor that has satisfies the
+            If ``True``, the beam search algorithm will return the first anchor that has satisfies the
             probability constraint.
         max_anchor_size
             Maximum number of features in result.
@@ -824,23 +845,26 @@ class AnchorTabular(Explainer, FitMixin):
             Min number of initial samples.
         n_covered_ex
             How many examples where anchors apply to store for each anchor sampled during search
-            (both examples where prediction on samples agrees/disagrees with desired_label are stored).
+            (both examples where prediction on samples agrees/disagrees with `desired_label` are stored).
         binary_cache_size
-            The result search pre-allocates binary_cache_size batches for storing the binary arrays
+            The result search pre-allocates `binary_cache_size` batches for storing the binary arrays
             returned during sampling.
         cache_margin
-            When only max(cache_margin, batch_size) positions in the binary cache remain empty, a new cache
+            When only ``max(cache_margin, batch_size)`` positions in the binary cache remain empty, a new cache
             of the same size is pre-allocated to continue buffering samples.
         verbose
             Display updates during the anchor search iterations.
         verbose_every
             Frequency of displayed iterations during anchor search process.
 
-
         Returns
         -------
         explanation
             `Explanation` object containing the result explaining the instance with additional metadata as attributes.
+            See usage at `AnchorTabular examples`_ for details.
+
+            .. _AnchorTabular examples:
+                https://docs.seldon.io/projects/alibi/en/latest/methods/Anchors.html
         """
         # transform one-hot encodings to labels if ohe == True
         X = ohe_to_ord(X_ohe=X.reshape(1, -1), cat_vars_ohe=self.cat_vars_ohe)[0].reshape(-1) if self.ohe else X
@@ -879,27 +903,27 @@ class AnchorTabular(Explainer, FitMixin):
         )  # type: Any
         self.mab = mab
 
-        return self.build_explanation(X, result, self.instance_label, params)
+        return self._build_explanation(X, result, self.instance_label, params)
 
-    def build_explanation(self, X: np.ndarray, result: dict, predicted_label: int, params: dict) -> Explanation:
+    def _build_explanation(self, X: np.ndarray, result: dict, predicted_label: int, params: dict) -> Explanation:
         """
         Preprocess search output and return an explanation object containing metdata
 
         Parameters
         ----------
-        X:
+        X
             Instance to be explained.
-        result:
+        result
             Dictionary with explanation search output and metadata.
-        predicted_label:
+        predicted_label
             Label of the instance to be explained (inferred if not given).
         params
-            Parameters passed to `explain`
+            Parameters passed to :py:meth:`alibi.explainers.anchor_tabular.AnchorTabular.explain`.
 
         Return
         ------
-             `Explanation` object containing human readable explanation, metadata, and precision/coverage
-             info as attributes.
+        `Explanation` object containing the anchor explaining the instance with additional metadata as attributes. \
+
         """
 
         self.add_names_to_exp(result)
@@ -999,23 +1023,30 @@ class AnchorTabular(Explainer, FitMixin):
             explanation['names'].append(fname)
 
     @property
-    def predictor(self) -> Callable:
+    def predictor(self) -> Optional[Callable]:  # because of saving.py setting it to None
         return self._ohe_predictor if self.ohe else self._predictor
 
     @predictor.setter
-    def predictor(self, predictor: Optional[Callable]) -> None:
-        # if input is one-hot encoded
-        if self.ohe:
-            # this predictor expects ordinal/labels encoded categorical variables
-            ord_predictor = lambda x: predictor(ord_to_ohe(x, self.cat_vars_ord)[0])  # noqa: E731
-            self._predictor = self._transform_predictor(ord_predictor) if predictor else None
-
-            # this predictor expects one-hot encoded categorical variable
-            self._ohe_predictor = self._transform_ohe_predictor(predictor) if predictor else None
-
+    def predictor(self, predictor: Optional[Callable]) -> None:  # Optional here because in saving.py we set it to None
+        # deal with the case from saving.py first
+        # TODO: how do we prevent users from passing predictor=None? Probably beartype.
+        if predictor is None:
+            self._predictor = None
+            if self.ohe:
+                self._ohe_predictor = None
         else:
-            # set the predictor
-            self._predictor = self._transform_predictor(predictor) if predictor else None
+            # if input is one-hot encoded
+            if self.ohe:
+                # this predictor expects ordinal/labels encoded categorical variables
+                ord_predictor = lambda x: predictor(ord_to_ohe(x, self.cat_vars_ord)[0])  # noqa: E731
+                self._predictor = self._transform_predictor(ord_predictor)
+
+                # this predictor expects one-hot encoded categorical variable
+                self._ohe_predictor = self._transform_ohe_predictor(predictor)
+
+            else:
+                # set the predictor
+                self._predictor = self._transform_predictor(predictor)
 
     def _transform_predictor(self, predictor: Callable) -> Callable:
         # define data instance full of zeros
@@ -1046,6 +1077,14 @@ class AnchorTabular(Explainer, FitMixin):
         return predictor
 
     def reset_predictor(self, predictor: Callable) -> None:
+        """
+        Resets the predictor function.
+
+        Parameters
+        ----------
+        predictor
+            New predictor function.
+        """
         self.predictor = predictor
         self.samplers[0].predictor = self._predictor
 
@@ -1053,23 +1092,33 @@ class AnchorTabular(Explainer, FitMixin):
 class DistributedAnchorTabular(AnchorTabular):
     if RAY_INSTALLED:
         import ray
-        ray = ray  # set module as class variable to used only in this context
+        # set module as class variable to used only in this context
+        ray = ray  #: `ray` module.
 
-    def __init__(self, predictor: Callable, feature_names: List[str], categorical_names: Dict[int, List[str]] = None,
-                 dtype: Type[np.generic] = np.float32, ohe: bool = False, seed: int = None) -> None:
+    def __init__(self,
+                 predictor: Callable,
+                 feature_names: List[str],
+                 categorical_names: Optional[Dict[int, List[str]]] = None,
+                 dtype: Type[np.generic] = np.float32,
+                 ohe: bool = False,
+                 seed: Optional[int] = None) -> None:
 
         super().__init__(predictor, feature_names, categorical_names, dtype, ohe, seed)
         if not DistributedAnchorTabular.ray.is_initialized():
             DistributedAnchorTabular.ray.init()
 
-    def fit(self, train_data: np.ndarray, disc_perc: tuple = (25, 50, 75), **kwargs) -> "AnchorTabular":  # type: ignore
+    def fit(self,  # type: ignore[override]
+            train_data: np.ndarray,
+            disc_perc: tuple = (25, 50, 75),
+            **kwargs) -> "AnchorTabular":
         """
         Creates a list of handles to parallel processes handles that are used for submitting sampling
         tasks.
 
         Parameters
         ----------
-            See superclass implementation.
+        train_data, disc_perc, **kwargs
+            See :py:meth:`alibi.explainers.anchor_tabular.AnchorTabular.fit` superclass.
         """
 
         try:
@@ -1097,7 +1146,7 @@ class DistributedAnchorTabular(AnchorTabular):
         )
         train_data_id = DistributedAnchorTabular.ray.put(train_data)
         d_train_data_id = DistributedAnchorTabular.ray.put(d_train_data)
-        samplers = [TabularSampler(*sampler_args, seed=self.seed) for _ in range(ncpu)]  # type: ignore
+        samplers = [TabularSampler(*sampler_args, seed=self.seed) for _ in range(ncpu)]  # type: ignore[arg-type]
         d_samplers = []
         for sampler in samplers:
             d_samplers.append(
@@ -1114,12 +1163,12 @@ class DistributedAnchorTabular(AnchorTabular):
 
     def _build_sampling_lookups(self, X: np.ndarray) -> None:
         """
-        See superclass documentation.
+        See :py:meth:`alibi.explainers.anchor_tabular.AnchorTabular._build_sampling_lookups` documentation.
 
         Parameters
         ----------
-        X:
-            See superclass documentation.
+        X
+            See :py:meth:`alibi.explainers.anchor_tabular.AnchorTabular._build_sampling_lookups` documentation.
         """
 
         lookups = [sampler.build_lookups.remote(X) for sampler in self.samplers][0]
@@ -1134,7 +1183,7 @@ class DistributedAnchorTabular(AnchorTabular):
                 coverage_samples: int = 10000,
                 beam_size: int = 1,
                 stop_on_first: bool = False,
-                max_anchor_size: int = None,
+                max_anchor_size: Optional[int] = None,
                 min_samples_start: int = 1,
                 n_covered_ex: int = 10,
                 binary_cache_size: int = 10000,
@@ -1143,16 +1192,18 @@ class DistributedAnchorTabular(AnchorTabular):
                 verbose_every: int = 1,
                 **kwargs: Any) -> Explanation:
         """
-        Explains the prediction made by a classifier on instance X. Sampling is done in parallel over a number of
-        cores specified in kwargs['ncpu'].
+        Explains the prediction made by a classifier on instance `X`. Sampling is done in parallel over a number of
+        cores specified in `kwargs['ncpu']`.
 
         Parameters
         ----------
-            See superclass implementation.
+        X, threshold, delta, tau, batch_size, coverage_samples, beam_size, stop_on_first, max_anchor_size, \
+        min_samples_start, n_covered_ex, binary_cache_size, cache_margin, verbose, verbose_every, **kwargs
+            See :py:meth:`alibi.explainers.anchor_tabular.AnchorTabular.explain`.
 
         Returns
         -------
-            See superclass implementation.
+        See :py:meth:`alibi.explainers.anchor_tabular.AnchorTabular.explain` superclass.
         """
         # transform one-hot encodings to labels if ohe == True
         X = ohe_to_ord(X_ohe=X.reshape(1, -1), cat_vars_ohe=self.cat_vars_ohe)[0].reshape(-1) if self.ohe else X
@@ -1191,9 +1242,17 @@ class DistributedAnchorTabular(AnchorTabular):
         )  # type: Any
         self.mab = mab
 
-        return self.build_explanation(X, result, self.instance_label, params)
+        return self._build_explanation(X, result, self.instance_label, params)
 
     def reset_predictor(self, predictor: Callable) -> None:
+        """
+        Resets the predictor function.
+
+        Parameters
+        ----------
+        predictor
+            New model prediction function.
+        """
         raise NotImplementedError("Resetting predictor is currently not supported for distributed explainers.")
         # TODO: to support resetting a predictor we would need to re-run most of the code in `fit` instantiating the
         # instances of RemoteSampler anew
