@@ -1,21 +1,23 @@
 import copy
-import spacy
-import string
 import logging
-import numpy as np
-import tensorflow as tf
-
+import string
+from abc import abstractmethod
 from copy import deepcopy
 from functools import partial
-from abc import abstractmethod
-from typing import Any, Callable, Dict, List, Tuple, TYPE_CHECKING, Union, Optional
+from typing import (TYPE_CHECKING, Any, Callable, Dict, List, Optional, Tuple,
+                    Type, Union)
 
-from alibi.utils.wrappers import ArgmaxTransformer
-from alibi.utils.lang_model import LanguageModel
+import numpy as np
+import spacy
+import tensorflow as tf
 
+from alibi.api.defaults import DEFAULT_DATA_ANCHOR, DEFAULT_META_ANCHOR
 from alibi.api.interfaces import Explainer, Explanation
-from alibi.api.defaults import DEFAULT_META_ANCHOR, DEFAULT_DATA_ANCHOR
-from alibi.exceptions import AlibiPredictorCallException, AlibiPredictorReturnTypeError
+from alibi.exceptions import (AlibiPredictorCallException,
+                              AlibiPredictorReturnTypeError)
+from alibi.utils.lang_model import LanguageModel
+from alibi.utils.wrappers import ArgmaxTransformer
+
 from .anchor_base import AnchorBaseBeam
 from .anchor_explanation import AnchorExplanation
 
@@ -24,7 +26,7 @@ if TYPE_CHECKING:
 logger = logging.getLogger(__name__)
 
 
-def _load_spacy_lexeme_prob(nlp: 'spacy.language.Language'):
+def _load_spacy_lexeme_prob(nlp: 'spacy.language.Language') -> 'spacy.language.Language':
     """
     This utility function loads the `lexeme_prob` table for a spacy model if it is not present.
     This is required to enable support for different spacy versions.
@@ -54,14 +56,13 @@ def _load_spacy_lexeme_prob(nlp: 'spacy.language.Language'):
         # https://github.com/explosion/spaCy/discussions/6388#discussioncomment-331096
         if 'lexeme_prob' not in nlp.vocab.lookups.tables:
             from spacy.lookups import load_lookups
-            lookups = load_lookups(nlp.lang, ['lexeme_prob'])
-            nlp.vocab.lookups.add_table('lexeme_prob', lookups.get_table('lexeme_prob'))  # type: ignore
+            lookups = load_lookups(nlp.lang, ['lexeme_prob'])  # type: ignore[arg-type]
+            nlp.vocab.lookups.add_table('lexeme_prob', lookups.get_table('lexeme_prob'))
 
     return nlp
 
 
-class Neighbors(object):
-
+class Neighbors:
     def __init__(self, nlp_obj: 'spacy.language.Language', n_similar: int = 500, w_prob: float = -15.) -> None:
         """
         Initialize class identifying neighbouring words from the embedding for a given word.
@@ -69,7 +70,7 @@ class Neighbors(object):
         Parameters
         ----------
         nlp_obj
-            spaCy model.
+            `spaCy` model.
         n_similar
             Number of similar words to return.
         w_prob
@@ -96,13 +97,12 @@ class Neighbors(object):
         tag
             Part of speech tag for the words.
         top_n
-            Return only top_n neighbors.
+            Return only `top_n` neighbors.
 
         Returns
         -------
-        A dict with two fields. The 'words' field contains a numpy array
-        of the top_n most similar words, whereas the fields similarity is
-        a numpy array with corresponding word similarities.
+        A dict with two fields. The ``'words'`` field contains a `numpy` array of the `top_n` most similar words, \
+        whereas the fields ``'similarities'`` is a `numpy` array with corresponding word similarities.
         """
 
         # the word itself is excluded so we add one to return the expected number of words
@@ -140,20 +140,20 @@ class AnchorTextSampler:
     def __call__(self, anchor: tuple, num_samples: int) -> Tuple[np.ndarray, np.ndarray]:
         pass
 
-    def _joiner(self, arr: np.ndarray, dtype: np.dtype = None) -> np.ndarray:
+    def _joiner(self, arr: np.ndarray, dtype: Optional[Type[np.generic]] = None) -> np.ndarray:
         """
-        Function to concatenate an np.array of strings along a specified axis.
+        Function to concatenate a `numpy` array of strings along a specified axis.
 
         Parameters
         ----------
         arr
-            1D numpy array of strings.
+            1D `numpy` array of strings.
         dtype
            Array type, used to avoid truncation of strings when concatenating along axis.
 
         Returns
         -------
-            Array with one element, the concatenation of the strings in the input array.
+        Array with one element, the concatenation of the strings in the input array.
         """
         if not dtype:
             return np.array(' '.join(arr))
@@ -162,7 +162,7 @@ class AnchorTextSampler:
 
 
 class UnknownSampler(AnchorTextSampler):
-    UNK = "UNK"
+    UNK: str = "UNK"  #: Unknown token to be used.
 
     def __init__(self, nlp: 'spacy.language.Language', perturb_opts: Dict):
         """
@@ -171,7 +171,7 @@ class UnknownSampler(AnchorTextSampler):
         Parameters
         ----------
         nlp
-            spaCy object.
+            `spaCy` object.
         perturb_opts
             Perturbation options.
         """
@@ -204,8 +204,8 @@ class UnknownSampler(AnchorTextSampler):
 
     def __call__(self, anchor: tuple, num_samples: int) -> Tuple[np.ndarray, np.ndarray]:
         """
-        The function returns  an np.array of num_samples where randomly chosen features,
-        except those in anchor, are replaced by self.UNK token.
+        The function returns a `numpy` array of `num_samples` where randomly chosen features,
+        except those in anchor, are replaced by ``'UNK'`` token.
 
         Parameters
         ----------
@@ -219,7 +219,7 @@ class UnknownSampler(AnchorTextSampler):
         raw
             Array containing num_samples elements. Each element is a perturbed sentence.
         data
-            A (num_samples, m)-dimensional boolean array, where m is the number of tokens
+            A `(num_samples, m)`-dimensional boolean array, where `m` is the number of tokens
             in the instance to be explained.
         """
         assert self.perturb_opts, "Perturbation options are not set."
@@ -248,7 +248,7 @@ class UnknownSampler(AnchorTextSampler):
 
     def set_data_type(self) -> None:
         """
-        Working with numpy arrays of strings requires setting the data type to avoid
+        Working with `numpy` arrays of strings requires setting the data type to avoid
         truncating examples. This function estimates the longest sentence expected
         during the sampling process, which is used to set the number of characters
         for the samples and examples arrays. This depends on the perturbation method
@@ -260,6 +260,7 @@ class UnknownSampler(AnchorTextSampler):
 
 
 class SimilaritySampler(AnchorTextSampler):
+
     def __init__(self, nlp: 'spacy.language.Language', perturb_opts: Dict):
         """
         Initialize similarity sampler. This sampler replaces words with similar words.
@@ -267,7 +268,7 @@ class SimilaritySampler(AnchorTextSampler):
         Parameters
         ----------
         nlp
-            spaCy object.
+            `spaCy` object.
         perturb_opts
             Perturbation options.
 
@@ -283,7 +284,10 @@ class SimilaritySampler(AnchorTextSampler):
 
         # dict containing an np.array of similar words with same part of speech and an np.array of similarities
         self.synonyms = {}  # type: Dict[str, Dict[str, np.ndarray]]
-        self.tokens, self.words, self.positions, self.punctuation = [], [], [], []  # type: List, List, List, List
+        self.tokens: 'spacy.tokens.Doc'
+        self.words: List[str] = []
+        self.positions: List[int] = []
+        self.punctuation: List['spacy.tokens.Token'] = []
 
     def set_text(self, text: str) -> None:
         """
@@ -292,7 +296,7 @@ class SimilaritySampler(AnchorTextSampler):
         Parameters
         ----------
         text
-          Text to be processed.
+            Text to be processed.
         """
         processed = self.nlp(text)  # spaCy tokens for text
         self.words = [x.text for x in processed]  # list with words in text
@@ -308,10 +312,10 @@ class SimilaritySampler(AnchorTextSampler):
 
     def find_similar_words(self) -> None:
         """
-        This function queries a spaCy nlp model to find n similar words with the same
+        This function queries a `spaCy` nlp model to find `n` similar words with the same
         part of speech for each word in the instance to be explained. For each word
-        the search procedure returns a dictionary containing an np.array of words ('words')
-        and an np.array of word similarities ('similarities').
+        the search procedure returns a dictionary containing a `numpy` array of words (``'words'``)
+        and a `numpy` array of word similarities (``'similarities'``).
         """
         for word, token in zip(self.words, self.tokens):
             if word not in self.synonyms:
@@ -319,9 +323,10 @@ class SimilaritySampler(AnchorTextSampler):
 
     def __call__(self, anchor: tuple, num_samples: int) -> Tuple[np.ndarray, np.ndarray]:
         """
-        The function returns an np.array of num_samples where randomly chosen features,
-        except those in anchor, are replaced by similar words with the same part of
-        speech of tag. See self.perturb_sentence for details of how the replacement works.
+        The function returns a `numpy` array of `num_samples` where randomly chosen features,
+        except those in anchor, are replaced by similar words with the same part of speech of tag.
+        See :py:meth:`alibi.explainers.anchor_text.SimilaritySampler.perturb_sentence_similarity` for details of how
+        the replacement works.
 
         Parameters
         ----------
@@ -332,7 +337,7 @@ class SimilaritySampler(AnchorTextSampler):
 
         Returns
         -------
-        See perturb_sentence_similarity
+        See :py:meth:`alibi.explainers.anchor_text.SimilaritySampler.perturb_sentence`.
         """
         assert self.perturb_opts, "Perturbation options are not set."
         return self.perturb_sentence_similarity(anchor, num_samples, **self.perturb_opts)
@@ -358,7 +363,7 @@ class SimilaritySampler(AnchorTextSampler):
         n
             Number of samples used when sampling from the corpus.
         sample_proba
-            Sample probability for a word if use_proba is False.
+            Sample probability for a word if `use_proba=False`.
         forbidden
             Forbidden lemmas.
         forbidden_tags
@@ -370,15 +375,16 @@ class SimilaritySampler(AnchorTextSampler):
         use_proba
             Bool whether to sample according to a similarity score with the corpus embeddings.
         temperature
-            Sample weight hyperparameter if use_proba equals True.
+            Sample weight hyper-parameter if ``use_proba=True``.
+        **kwargs
+            Other arguments. Not used.
 
         Returns
         -------
         raw
             Array of perturbed text instances.
         data
-            Matrix with 1s and 0s indicating whether a word in the text
-            has not been perturbed for each sample.
+            Matrix with 1s and 0s indicating whether a word in the text has not been perturbed for each sample.
         """
         # allocate memory for the binary mask and the perturbed instances
         raw = np.zeros((n, len(self.tokens)), self.dtype)
@@ -420,7 +426,7 @@ class SimilaritySampler(AnchorTextSampler):
 
     def set_data_type(self) -> None:
         """
-        Working with numpy arrays of strings requires setting the data type to avoid
+        Working with `numpy` arrays of strings requires setting the data type to avoid
         truncating examples. This function estimates the longest sentence expected
         during the sampling process, which is used to set the number of characters
         for the samples and examples arrays. This depends on the perturbation method
@@ -439,15 +445,15 @@ class SimilaritySampler(AnchorTextSampler):
 
 class LanguageModelSampler(AnchorTextSampler):
     # filling procedures
-    FILLING_PARALLEL = 'parallel'
-    FILLING_AUTOREGRESSIVE = 'autoregressive'
+    FILLING_PARALLEL: str = 'parallel'  #: Parallel filling procedure.
+    FILLING_AUTOREGRESSIVE = 'autoregressive'  #: Autoregressive filling procedure. Considerably slow.
 
     def __init__(self, model: LanguageModel, perturb_opts: dict, ):
         """
         Initialize language model sampler. This sampler replaces words with the ones
         sampled according to the output distribution of the language model. There are
-        two modes to use the sampler: `parallel` and `autoregressive`. In the parallel
-        mode, all words are replaced simultaneously. In the `autoregressive` model, the words
+        two modes to use the sampler: ``'parallel'`` and ``'autoregressive'``. In the ``'parallel'``
+        mode, all words are replaced simultaneously. In the ``'autoregressive'`` model, the words
         are replaced one by one, starting from left to right. Thus the following words
         are conditioned on the previous predicted words.
 
@@ -502,6 +508,8 @@ class LanguageModelSampler(AnchorTextSampler):
             String of punctuation characters.
         stopwords
             List of stopwords.
+        **kwargs
+            Other arguments. Not used.
         """
         # transform stopwords to lowercase
         if stopwords:
@@ -562,7 +570,7 @@ class LanguageModelSampler(AnchorTextSampler):
 
     def __call__(self, anchor: tuple, num_samples: int) -> Tuple[np.ndarray, np.ndarray]:
         """
-        The function returns an np.array of num_samples where randomly chosen features,
+        The function returns a `numpy` array of `num_samples` where randomly chosen features,
         except those in anchor, are replaced by words sampled according to the language
         model's predictions.
 
@@ -575,7 +583,7 @@ class LanguageModelSampler(AnchorTextSampler):
 
         Returns
         -------
-        See perturb_sentence
+        See :py:meth:`alibi.explainers.anchor_text.LanguageModelSampler.perturb_sentence`.
         """
         assert self.perturb_opts, "Perturbation options are not set."
         return self.perturb_sentence(anchor, num_samples, **self.perturb_opts)
@@ -589,7 +597,7 @@ class LanguageModelSampler(AnchorTextSampler):
                          filling: str = "parallel",
                          **kwargs) -> Tuple[np.ndarray, np.ndarray]:
         """
-        The function returns an np.array of num_samples where randomly chosen features,
+        The function returns an `numpy` array of `num_samples` where randomly chosen features,
         except those in anchor, are replaced by words sampled according to the language
         model's predictions.
 
@@ -606,14 +614,16 @@ class LanguageModelSampler(AnchorTextSampler):
         batch_size_lm:
             Batch size used for language model.
         filling:
-            Method to fill masked words. Either `parallel` or `autoregressive`.
+            Method to fill masked words. Either ``'parallel'`` or ``'autoregressive'``.
+        **kwargs
+            Other arguments to be passed to other methods.
 
         Returns
         -------
         raw
-            Array containing num_samples elements. Each element is a perturbed sentence.
+            Array containing `num_samples` elements. Each element is a perturbed sentence.
         data
-            A (num_samples, m)-dimensional boolean array, where m is the number of tokens
+            A `(num_samples, m)`-dimensional boolean array, where `m` is the number of tokens
             in the instance to be explained.
         """
         # Create the mask
@@ -660,16 +670,18 @@ class LanguageModelSampler(AnchorTextSampler):
         sample_proba
             Probability of a word being replaced.
         filling:
-            Method to fill masked words. Either `parallel` or `autoregressive`.
+            Method to fill masked words. Either ``'parallel'`` or ``'autoregressive'``.
         frac_mask_templates
             Fraction of mask templates from the number of requested samples.
+        **kwargs
+            Other arguments to be passed to other methods.
 
         Returns
         -------
         raw
             Array with masked instances.
         data
-            A (num_samples, m)-dimensional boolean array, where m is the number of tokens
+            A `(num_samples, m)`-dimensional boolean array, where `m` is the number of tokens
             in the instance to be explained.
         """
         # make sure that frac_mask_templates is in [0, 1]
@@ -750,20 +762,20 @@ class LanguageModelSampler(AnchorTextSampler):
         # convert to array and return
         return np.array(full_raw, dtype=self.dtype_sent)
 
-    def _joiner(self, arr: np.ndarray, dtype: np.dtype = None) -> np.ndarray:
+    def _joiner(self, arr: np.ndarray, dtype: Optional[Type[np.generic]] = None) -> np.ndarray:
         """
-        Function to concatenate an np.array of strings along a specified axis.
+        Function to concatenate an `numpy` array of strings along a specified axis.
 
         Parameters
         ----------
         arr
-            1D numpy array of strings.
+            1D `numpy` array of strings.
         dtype
-           Array type, used to avoid truncation of strings when concatenating along axis.
+            Array type, used to avoid truncation of strings when concatenating along axis.
 
         Returns
         -------
-            Array with one element, the concatenation of the strings in the input array.
+        Array with one element, the concatenation of the strings in the input array.
         """
         filtered_arr = list(filter(lambda x: len(x) > 0, arr))
         str_arr = self.model.tokenizer.convert_tokens_to_string(filtered_arr)
@@ -789,8 +801,7 @@ class LanguageModelSampler(AnchorTextSampler):
         raw
             Array of mask templates.
         data
-            Binary mask having 0 where the word was masked. Note that
-            the width of the data array is equal to the length of id_samples.
+            Binary mask having 0 where the word was masked.
         num_samples
             Number of samples to be drawn.
         top_n:
@@ -798,12 +809,14 @@ class LanguageModelSampler(AnchorTextSampler):
         batch_size_lm:
             Batch size used for language model.
         filling
-            Method to fill masked words. Either `parallel` or `autoregressive`.
+            Method to fill masked words. Either ``'parallel'`` or ``'autoregressive'``.
+        **kwargs
+            Other paremeters to be passed to other methods.
 
         Returns
         -------
         raw
-            Array containing num_samples elements. Each element is a perturbed sentence.
+            Array containing `num_samples` elements. Each element is a perturbed sentence.
         """
         # chose the perturbation function
         perturb_func = self._perturb_instances_parallel if filling == self.FILLING_PARALLEL \
@@ -821,10 +834,10 @@ class LanguageModelSampler(AnchorTextSampler):
 
     def _remove_subwords(self, raw: np.ndarray, row: int, col: int, punctuation: str = '', **kwargs) -> np.ndarray:
         """
-        Deletes the subwords that follow a given token identified by the (row, col) pair in the `raw` matrix.
+        Deletes the subwords that follow a given token identified by the `(row, col)` pair in the `raw` matrix.
         A token is considered to be part of a word if is not a punctuation and if has the subword prefix
         specific to the used language model. The subwords are not actually deleted in, but they are replace
-        by the empty string ``
+        by the empty string ``''``.
 
         Parameters
         ----------
@@ -883,15 +896,15 @@ class LanguageModelSampler(AnchorTextSampler):
             Sample weight hyper-parameter.
         use_proba
             Bool whether to sample according to the predicted words distribution
+        **kwargs
+            Other arguments. Not used.
 
         Returns
         -------
         sampled_tokens
-            Tensor containing the ids of the sampled tokens.
-            Has `num_samples` rows.
+            Array containing the ids of the sampled tokens. Has `num_samples` rows.
         sampled_data
-            Binary array having 0 where the tokens were masked.
-            Has `num_samples` rows.
+            Binary array having 0 where the tokens were masked. Has `num_samples` rows.
         """
         # tokenize instances
         tokens_plus = self.model.tokenizer.batch_encode_plus(list(raw), padding=True, return_tensors='tf')
@@ -967,6 +980,7 @@ class LanguageModelSampler(AnchorTextSampler):
         """
         Perturb the instances in an autoregressive fashion (sequential).
 
+        Parameters
         ----------
         num_samples
             Number of samples to be generated.
@@ -982,15 +996,15 @@ class LanguageModelSampler(AnchorTextSampler):
             Sample weight hyper-parameter.
         use_proba
             Bool whether to sample according to the predicted words distribution.
+        **kwargs
+            Other arguments. Not used.
 
         Returns
         -------
         sampled_tokens
-            Tensor containing the ids of the sampled tokens.
-            Has `num_samples` rows.
+            Array containing the ids of the sampled tokens. Has `num_samples` rows.
         sampled_data
-            Binary array havin 0 where the tokens were masked.
-            Has `num_samples` rows.
+            Binary array having 0 where the tokens were masked. Has `num_samples` rows.
         """
         # number of samples to generate per mask template
         assert num_samples == raw.shape[0]
@@ -1052,7 +1066,7 @@ class LanguageModelSampler(AnchorTextSampler):
 
     def set_data_type(self) -> None:
         """
-        Working with numpy arrays of strings requires setting the data type to avoid
+        Working with `numpy` arrays of strings requires setting the data type to avoid
         truncating examples. This function estimates the longest sentence expected
         during the sampling process, which is used to set the number of characters
         for the samples and examples arrays. This depends on the perturbation method
@@ -1083,9 +1097,9 @@ DEFAULT_SAMPLING_UNKNOWN = {
     "sample_proba": 0.5
 }
 """
-Default perturbation options for `unknown` sampling
+Default perturbation options for ``'unknown'`` sampling
 
-    - ``'sample_proba'``: float, probability of a word to be masked.
+    - ``'sample_proba'`` : ``float`` - probability of a word to be masked.
 """
 
 DEFAULT_SAMPLING_SIMILARITY = {
@@ -1095,15 +1109,15 @@ DEFAULT_SAMPLING_SIMILARITY = {
     "use_proba": False
 }
 """
-Default perturbation options for `similarity` sampling
+Default perturbation options for ``'similarity'`` sampling
 
-    - ``'sample_proba'``: float, probability of a word to be masked.
+    - ``'sample_proba'`` : ``float`` - probability of a word to be masked.
 
-    - ``'top_n'``: int, number of similar words to sample for perturbations
+    - ``'top_n'`` : ``int`` - number of similar words to sample for perturbations.
 
-    - ``'temperature'``: float, sample weight hyper-parameter if `use_proba` equals `True`.
+    - ``'temperature'`` : ``float`` - sample weight hyper-parameter if `use_proba=True`.
 
-    - ``'use_proba'``: bool, whether to sample according to the words similarity.
+    - ``'use_proba'`` : ``bool`` - whether to sample according to the words similarity.
 """
 
 DEFAULT_SAMPLING_LANGUAGE_MODEL = {
@@ -1119,48 +1133,48 @@ DEFAULT_SAMPLING_LANGUAGE_MODEL = {
     "sample_punctuation": False,
 }
 """
-Default perturbation options for `similarity` sampling
+Default perturbation options for ``'language_model'`` sampling
 
-    - ``'filling'``: str, filling method for language models. Allowed values: `parallel`, `autoregressive`. \
-    `parallel` method corresponds to a single forward pass through the language model. The masked words are sampled \
-    independently, according to the selected probability distribution (see `top_n`, `temperature`, `use_proba`).  \
-    `autoregressive` method fills the words one at the time. This corresponds to multiple forward passes through  \
-    the language model which is computationally expensive.
+    - ``'filling'`` : ``str`` - filling method for language models. Allowed values: ``'parallel'``, \
+    ``'autoregressive'``. ``'parallel'`` method corresponds to a single forward pass through the language model. The \
+    masked words are sampled independently, according to the selected probability distribution (see `top_n`, \
+    `temperature`, `use_proba`). `autoregressive` method fills the words one at the time. This corresponds to \
+    multiple forward passes through  the language model which is computationally expensive.
 
-    - ``'sample_proba'``: float, probability of a word to be masked.
+    - ``'sample_proba'`` : ``float`` - probability of a word to be masked.
 
-    - ``'top_n'``: int, number of similar words to sample for perturbations.
+    - ``'top_n'`` : ``int`` - number of similar words to sample for perturbations.
 
-    - ``'temperature'``: float, sample weight hyper-parameter if use_proba equals True.
+    - ``'temperature'`` : ``float`` - sample weight hyper-parameter if use_proba equals ``True``.
 
-    - ``'use_proba'``: bool, whether to sample according to the predicted words distribution. If set to `False`, \
-    the `top_n` words are sampled uniformly at random.
+    - ``'use_proba'`` : ``bool`` - whether to sample according to the predicted words distribution. If set to \
+    ``False``, the `top_n` words are sampled uniformly at random.
 
-    - ``frac_mask_template'``: float, fraction from the number of samples of mask templates to be generated. \
-    In each sampling call, will generate int(`frac_mask_templates` * `num_samples`) masking templates. \
+    - ``'frac_mask_template'`` : ``float`` - fraction from the number of samples of mask templates to be generated. \
+    In each sampling call, will generate `int(frac_mask_templates * num_samples)` masking templates. \
     Lower fraction corresponds to lower computation time since the batch fed to the language model is smaller. \
     After the words' distributions is predicted for each mask, a total of `num_samples` will be generated by sampling \
-    evenly from each template. Note that lower fraction might correspond to less diverse sample. A `sample_proba` \
-    is `1` corresponds to masking each word. For this case only one masking template will be constructed. \
-    A `filling` set to `autoregressive` will generate `num_samples` masking templates regardless of the value \
+    evenly from each template. Note that lower fraction might correspond to less diverse sample. A `sample_proba=1` \
+    corresponds to masking each word. For this case only one masking template will be constructed. \
+    A `filling='autoregressive'` will generate `num_samples` masking templates regardless of the value \
     of `frac_mask_templates`.
 
-    - ``batch_size_lm``: int, batch size used for the language model forward pass.
+    - ``'batch_size_lm'`` : ``int`` - batch size used for the language model forward pass.
 
-    - ``punctuation``: str, string of punctuation not to be masked.
+    - ``'punctuation'`` : ``str`` - string of punctuation not to be masked.
 
-    - ``stopwords``: List[str], list of words not to be masked.
+    - ``'stopwords'`` : ``List[str]`` - list of words not to be masked.
 
-    - ``sample_punctuation``: bool, whether to sample punctuation to fill the masked words. If `False`, the \
+    - ``'sample_punctuation'`` : ``bool`` - whether to sample punctuation to fill the masked words. If ``False``, the \
     punctuation defined in `punctuation` will not be sampled.
 """
 
 
 class AnchorText(Explainer):
     # sampling methods
-    SAMPLING_UNKNOWN = 'unknown'
-    SAMPLING_SIMILARITY = 'similarity'
-    SAMPLING_LANGUAGE_MODEL = 'language_model'
+    SAMPLING_UNKNOWN = 'unknown'  #: Unknown sampling strategy.
+    SAMPLING_SIMILARITY = 'similarity'  #: Similarity sampling strategy.
+    SAMPLING_LANGUAGE_MODEL = 'language_model'  #: Language model sampling strategy.
 
     # default params
     DEFAULTS: Dict[str, Dict] = {
@@ -1189,25 +1203,31 @@ class AnchorText(Explainer):
         Parameters
         ----------
         predictor
-            A callable that takes a list of text strings representing N data points as inputs and returns N outputs.
+            A callable that takes a list of text strings representing `N` data points as inputs and returns `N` outputs.
         sampling_strategy
-            Perturbation distribution method.
-            `unknown` - replaces words with UNKs.
-            `similarity` - samples according to a similarity score with the corpus embeddings.
-            `language_model` - samples according the language model's output distributions.
+            Perturbation distribution method:
+
+             - ``'unknown'`` - replaces words with UNKs.
+
+             - ``'similarity'`` - samples according to a similarity score with the corpus embeddings.
+
+             - ``'language_model'`` - samples according the language model's output distributions.
+
         nlp
-            spaCy object when sampling method is `unknown` or `similarity`.
+            `spaCy` object when sampling method is ``'unknown'`` or ``'similarity'``.
         language_model
             Transformers masked language model. This is a model that it adheres to the
             `LanguageModel` interface we define in :py:class:`alibi.utils.lang_model.LanguageModel`.
         seed
             If set, ensure identical random streams.
         kwargs
-            Sampling parameters can be passed as `kwargs` depending on the `sampling_strategy`.
-            Check default parameters defined in:
+            Sampling arguments can be passed as `kwargs` depending on the `sampling_strategy`.
+            Check default arguments defined in:
 
                 - :py:data:`alibi.explainers.anchor_text.DEFAULT_SAMPLING_UNKNOWN`
+
                 - :py:data:`alibi.explainers.anchor_text.DEFAULT_SAMPLING_SIMILARITY`
+
                 - :py:data:`alibi.explainers.anchor_text.DEFAULT_SAMPLING_LANGUAGE_MODEL`
 
         Raises
@@ -1225,14 +1245,15 @@ class AnchorText(Explainer):
 
         # define model which can be either spacy object or LanguageModel
         # the initialization of the model happens in _validate_kwargs
-        self.model: Union['spacy.language.Language', LanguageModel]
+        self.model: Union['spacy.language.Language', LanguageModel]  #: Language model to be used.
 
         # validate kwargs
         self.perturb_opts, all_opts = self._validate_kwargs(sampling_strategy=sampling_strategy, nlp=nlp,
                                                             language_model=language_model, **kwargs)
 
         # set perturbation
-        self.perturbation: Any = self.CLASS_SAMPLER[self.sampling_strategy](self.model, self.perturb_opts)
+        self.perturbation: Any = \
+            self.CLASS_SAMPLER[self.sampling_strategy](self.model, self.perturb_opts)  #: Perturbation method.
 
         # update metadata
         self.meta['params'].update(seed=seed)
@@ -1304,28 +1325,36 @@ class AnchorText(Explainer):
         Parameters
         ----------
         anchor
-            int: the position of the anchor in the input batch
-            tuple: the anchor itself, a list of words to be kept unchanged
+             - ``int`` - the position of the anchor in the input batch.
+
+             - ``tuple`` - the anchor itself, a list of words to be kept unchanged.
+
         num_samples
             Number of generated perturbed samples.
         compute_labels
-            If True, an array of comparisons between predictions on perturbed samples and
+            If ``True``, an array of comparisons between predictions on perturbed samples and
             instance to be explained is returned.
 
         Returns
         -------
-            If compute_labels=True, a list containing the following is returned:
-             - covered_true: perturbed examples where the anchor applies and the model prediction
-                    on perturbation is the same as the instance prediction
-             - covered_false: perturbed examples where the anchor applies and the model prediction
-                    is NOT the same as the instance prediction
-             - labels: num_samples ints indicating whether the prediction on the perturbed sample
-                    matches (1) the label of the instance to be explained or not (0)
-             - data: Matrix with 1s and 0s indicating whether a word in the text has been
-                     perturbed for each sample
-             - 1.0: indicates exact coverage is not computed for this algorithm
-             - anchor[0]: position of anchor in the batch request
-            Otherwise, a list containing the data matrix only is returned.
+        If ``compute_labels=True``, a list containing the following is returned
+
+         - `covered_true` - perturbed examples where the anchor applies and the model prediction \
+         on perturbation is the same as the instance prediction.
+
+         - `covered_false` - perturbed examples where the anchor applies and the model prediction \
+         is NOT the same as the instance prediction.
+
+         - `labels` - num_samples ints indicating whether the prediction on the perturbed sample \
+         matches (1) the label of the instance to be explained or not (0).
+
+         - `data` - Matrix with 1s and 0s indicating whether a word in the text has been perturbed for each sample.
+
+         - `-1.0` - indicates exact coverage is not computed for this algorithm.
+
+         - `anchor[0]` - position of anchor in the batch request.
+
+        Otherwise, a list containing the data matrix only is returned.
         """
 
         raw_data, data = self.perturbation(anchor[1], num_samples)
@@ -1354,11 +1383,11 @@ class AnchorText(Explainer):
 
         Returns
         -------
-            A boolean array indicating whether the prediction was the same as the instance label.
+        A `numpy` boolean array indicating whether the prediction was the same as the instance label.
         """
         return self.predictor(samples.tolist()) == self.instance_label
 
-    def explain(self,  # type: ignore
+    def explain(self,  # type: ignore[override]
                 text: str,
                 threshold: float = 0.95,
                 delta: float = 0.1,
@@ -1367,7 +1396,7 @@ class AnchorText(Explainer):
                 coverage_samples: int = 10000,
                 beam_size: int = 1,
                 stop_on_first: bool = True,
-                max_anchor_size: int = None,
+                max_anchor_size: Optional[int] = None,
                 min_samples_start: int = 100,
                 n_covered_ex: int = 10,
                 binary_cache_size: int = 10000,
@@ -1385,7 +1414,7 @@ class AnchorText(Explainer):
         threshold
             Minimum precision threshold.
         delta
-            Used to compute beta.
+            Used to compute `beta`.
         tau
             Margin between lower confidence bound and minimum precision or upper bound.
         batch_size
@@ -1395,7 +1424,7 @@ class AnchorText(Explainer):
         beam_size
             Number of options kept after each stage of anchor building.
         stop_on_first
-            If True, the beam search algorithm will return the first anchor that has satisfies the
+            If ``True``, the beam search algorithm will return the first anchor that has satisfies the
             probability constraint.
         max_anchor_size
             Maximum number of features to include in an anchor.
@@ -1405,22 +1434,29 @@ class AnchorText(Explainer):
             How many examples where anchors apply to store for each anchor sampled during search
             (both examples where prediction on samples agrees/disagrees with predicted label are stored).
         binary_cache_size
-            The anchor search pre-allocates binary_cache_size batches for storing the boolean arrays
+            The anchor search pre-allocates `binary_cache_size` batches for storing the boolean arrays
             returned during sampling.
         cache_margin
-            When only max(cache_margin, batch_size) positions in the binary cache remain empty, a new cache
+            When only ``max(cache_margin, batch_size)`` positions in the binary cache remain empty, a new cache
             of the same size is pre-allocated to continue buffering samples.
-        kwargs
-            Other keyword arguments passed to the anchor beam search and the text sampling and perturbation functions.
         verbose
             Display updates during the anchor search iterations.
         verbose_every
             Frequency of displayed iterations during anchor search process.
+        **kwargs
+            Other keyword arguments passed to the anchor beam search and the text sampling and perturbation functions.
 
         Returns
         -------
-        explanation
-            `Explanation` object containing the anchor explaining the instance with additional metadata as attributes.
+        `Explanation` object containing the anchor explaining the instance with additional metadata as attributes. \
+        Contains the following data-related attributes
+
+         - `anchor` : ``List[str]`` - a list of words in the proposed anchor.
+
+         - `precision` : ``float`` - the fraction of times the sampled instances where the anchor holds yields \
+         the same prediction as the original instance. The precision will always be  threshold for a valid anchor.
+
+         - `coverage` : ``float`` - the fraction of sampled instances the anchor applies to.
         """
         # get params for storage in meta
         params = locals()
@@ -1476,10 +1512,11 @@ class AnchorText(Explainer):
 
         # set mab
         self.mab = mab
-        return self.build_explanation(text, result, self.instance_label, params)
+        return self._build_explanation(text, result, self.instance_label, params)
 
-    def build_explanation(self, text: str, result: dict, predicted_label: int, params: dict) -> Explanation:
-        """ Uses the metadata returned by the anchor search algorithm together with
+    def _build_explanation(self, text: str, result: dict, predicted_label: int, params: dict) -> Explanation:
+        """
+        Uses the metadata returned by the anchor search algorithm together with
         the instance to be explained to build an explanation object.
 
         Parameters
@@ -1491,7 +1528,7 @@ class AnchorText(Explainer):
         predicted_label
             Label of the instance to be explained. Inferred if not received.
         params
-            Parameters passed to `explain`.
+            Arguments passed to `explain`.
         """
 
         result['instance'] = text
@@ -1535,6 +1572,14 @@ class AnchorText(Explainer):
             return transformer
 
     def reset_predictor(self, predictor: Callable) -> None:
+        """
+        Resets the predictor function.
+
+        Parameters
+        ----------
+        predictor
+            New predictor function.
+        """
         self.predictor = self._transform_predictor(predictor)
 
     def _seed(self, seed: int) -> None:
